/*****************************************************************************

Xprotolab - AVR XMEGA Oscilloscope and Development Kit

Gabotronics C.A.
January 2011

www.gabotronics.com
email me at: gabriel@gabotronics.com

*****************************************************************************/

#include <stdint.h>
#include <avr/interrupt.h>
#include "logic.h"
#include "main.h"
#include "mso.h"

DCHANNEL CHD;                       // Digital Channel

//    0   1   2   3   4   5   6   7
//  |...|...|...|...|...|...|...|...|
//   ^
//   |
//   sample bit 3 times
//
// Display the HEX value of the digital stream
void HEXSerial(void) {
    int8_t start,end,i;
    uint8_t data,d=0,temp5,temp6,temp7;
    int16_t increment,counter;
    uint8_t result=0;
    // The vertical cursors define the section to decode
    start = VcursorA;
    end   = VcursorB;
    // If the vertical cursors are off, use the entire screen
    if(!testbit(Mcursor, cursorv)) {
        start = 0;
        end   = 127;
    }
    if(VcursorA<VcursorB) {
        increment = (end-start)<<3;
    }
    else {
        increment = -((start-end)<<3);
    }
    if(srate>11) {
        increment*=2;
        start=start<<1;
    }
    temp5 = CHD.position;
    for(temp6=128, temp7=7; temp6; temp6=temp6>>1, temp7--) {
        counter = (start<<8);
        if(srate<=11) counter+=(HPos<<8);
        counter+=increment;
        if(CHD.mask&temp6) {
            for(i=1; i<=32; i++) {
                data = CHD.data[hibyte(counter)];
                if(CHD.ctrl.invert) data=~data;
                if(!(i&0x03)) { // Time to check bits when i is multiple of 4
                    result<<=1;
                    if(d>1) result+=1;
                    d=0;
                }
                else {
                    if(testbit(data,temp7)) d++;
                }
                counter+=increment;
            }
            lcd_goto(120,temp5>>3);
            printhex(result);
            temp5+=8;
        }
    }
}

// Data indexes
volatile uint16_t indrx,indtx;
volatile uint8_t *ack_ptr;          // pointer to acknowledge bytes (packed bits)
volatile uint8_t *addr_ptr;         // pointer to address or data (packed bits)
volatile uint8_t *queue_ptr;
volatile uint8_t ack_bit;           // counter for keeping track of all bits
volatile uint8_t addr_bit;          // counter for keeping track of all bits

uint16_t baud;

/* UART Baud rates:
1:   4800
2:   9600
3:  19200
4:  38400
5:  57600
6: 115200
*/
const uint8_t BAUDA[8] PROGMEM = {
    3,      // 1200
    1,      // 2400
    253,    // 4800
    245,    // 9600
    229,    // 19200
    197,    // 38400
    110,    // 57600
    23,      // 115200
};

const uint8_t BAUDB[8] PROGMEM = {
    253,    // 1200
    237,    // 2400
    220,    // 4800
    204,    // 9600
    188,    // 19200
    172,    // 38400
    168,    // 57600
    164     // 115200
};

const uint16_t TCC0val[8] PROGMEM = {
    26665,  // 1200
    13332,  // 2400
    6665,   // 4800
    3332,   // 9600
    1666,   // 19200
    832,    // 38400
    554,    // 57600
    277,    // 115200
};

// Protocol Sniffer
void Sniffer(void) {
    uint16_t i;
    uint8_t addrdata, ack, data;
    uint8_t page=0; // 16 pages to display 1024 bytes of data
                    // one page has 8 lines, each line will display
                    // 8 bytes in this format: "FF+ "
    clr_display();
    lcd_goto(0,0);
    indrx=0; indtx=0;
    // Setup
    PR.PRPC  = 0x00;    // Enable PORTC peripherals
    DMA.CH0.ADDRCTRL  = 0b10000101;         // reload source addr after each burst, incr des, reload dest end block
    DMA.CH0.TRFCNT    = 640;                // buffer size
    DMA.CH0.DESTADDR0 = (((uint32_t) bigtemp)>>0*8) & 0xFF;
    DMA.CH0.DESTADDR1 = (((uint32_t) bigtemp)>>1*8) & 0xFF;
    DMA.CH0.DESTADDR2 = (((uint32_t) bigtemp)>>2*8) & 0xFF;
    if(CHD.ctrl3.i2c) {
        lcd_putsp(PSTR("I2C SNIFFER - BIT0:SDA BIT1:SCL"));
        addr_ptr=CHD.data;
        ack_ptr=CHD.data+128;
        queue_ptr=bigtemp;
        ack_bit = 0x80;      // counter for keeping track of all bits
        addr_bit = 0x80;     // counter for keeping track of all bits
        PORTC.PIN0CTRL = (PORTC.PIN0CTRL & 0b11111000) | 0b00000010;    // Sense falling edge
        PORTC.INTFLAGS = 0x03;  // Clear flags
        PORTC.INTCTRL   = 0x02; // Enable PortC INT 0
    }
    else if(CHD.ctrl3.rs232) {
        lcd_putsp(PSTR("UART SNIFFER - BIT2:RX BIT3:TX"));
        // Initialize USART for RX
        USARTC0.BAUDCTRLA = pgm_read_byte_near(BAUDA+CHD.ctrl2.baud);
        USARTC0.BAUDCTRLB = pgm_read_byte_near(BAUDB+CHD.ctrl2.baud);
        USARTC0.CTRLC = 0x03;       // Async, No Parity, 1 stop bit, 8 data bits
        if(CHD.ctrl2.parmode) {     // Check parity
            if(CHD.ctrl2.parity) USARTC0.CTRLC |= 0x30;
            else USARTC0.CTRLC |= 0x20;
        }
        if(CHD.ctrl2.stopbit) USARTC0.CTRLC |= 0x08;    // Second stop bit
        USARTC0.CTRLB = 0x10;       // Enable RX
        TCC0.CTRLA = 0x01;  // DIV1
        baud = (uint16_t)pgm_read_word_near(TCC0val+CHD.ctrl2.baud);
        TCC0.PER = baud/4; // Sample at 4 times the baud rate
        TCC0.INTCTRLA = 0x02;
        DMA.CH0.TRIGSRC   = 0x4B;               // USARTC0 is trigger source
        DMA.CH0.SRCADDR0  = (((uint32_t)(&USARTC0.DATA))>>0*8) & 0xFF;
        DMA.CH0.SRCADDR1  = (((uint32_t)(&USARTC0.DATA))>>1*8) & 0xFF;
        DMA.CH0.SRCADDR2  = (((uint32_t)(&USARTC0.DATA))>>2*8) & 0xFF;
        DMA.CH0.CTRLA     = 0b10100100;         // repeat, 1 byte burst 
    }
    else if(CHD.ctrl3.spi) {
        lcd_putsp(PSTR("SPI SNIFFER"));
        lcd_goto(0,2); lcd_putsp(PSTR("BIT4:/SS"));
        lcd_goto(0,3); lcd_putsp(PSTR("BIT5:MOSI"));
        lcd_goto(0,4); lcd_putsp(PSTR("BIT6:MISO"));
        lcd_goto(0,5); lcd_putsp(PSTR("BIT7:SCK"));
        PORTC.INTFLAGS = 0x03;  // Clear flags
        PORTC.INTCTRL   = 0x08; // Enable PortC INT 1
        i=0;
        if(CHD.ctrl2.CPOL) {    // CPOL == 1
            PORTC.PIN7CTRL |= PORT_INVEN_bm;
            i=0x08;
        }
        else {                  // CPOL == 0
            PORTC.PIN7CTRL &= ~PORT_INVEN_bm;
        }
        if(CHD.ctrl2.CPHA) i+=0x04; // CPHA == 1
        SPIC.CTRL = 0b01000000 + i;
        DMA.CH0.TRIGSRC   = 0x4A;               // SPIC is trigger source
        DMA.CH0.SRCADDR0  = (((uint32_t)(&SPIC.DATA))>>0*8) & 0xFF;
        DMA.CH0.SRCADDR1  = (((uint32_t)(&SPIC.DATA))>>1*8) & 0xFF;
        DMA.CH0.SRCADDR2  = (((uint32_t)(&SPIC.DATA))>>2*8) & 0xFF;
        DMA.CH0.CTRLA     = 0b10100100;         // repeat, 1 byte burst 
    }
    show_display();
    while((indrx==0) && (indtx==0)) {
        if (key==KD) goto exit;
        if(!CHD.ctrl3.i2c) indrx=640-DMA.CH0.TRFCNT;
    }
    do {
        show_display();
        while(!testbit(TCD1.INTFLAGS, TC1_OVFIF_bp)) {   // wait for refresh timeout
            if(testbit(Signals, userinput))
                break;       // cancel if user input
        }
        setbit(TCD1.INTFLAGS, TC1_OVFIF_bp);
        clr_display();
        // Display I2C data
        if(CHD.ctrl3.i2c) {
            lcd_goto(0,0);
            for(i=(uint16_t)page*64; (i<((uint16_t)page+1)*64); i++) {
                if(i>=indrx) break;
                data=bigtemp[i];
                addrdata=CHD.data[i/8];
                addrdata= (addrdata<<(i&0x0007))&0x80;
                ack=CHD.data[(i/8)+128];
                ack= (ack<<(i&0x0007))&0x80;
                if(addrdata) {  // Address
/*                    if(u8CursorX!=0) {    // Start Message in new line
                        u8CursorX=0;
                        u8CursorY++;
                    }*/
                    printhex(data>>1);
                    if(testbit(data,0)) {   // Read
                        if(ack) GLCD_Putchar('<');  // Ack
                        else    GLCD_Putchar('(');  // No Ack
                    }
                    else {                  // Write
                        if(ack) GLCD_Putchar('>');  // Ack
                        else    GLCD_Putchar(')');  // No Ack
                    }
                }
                else {          // Data
                    printhex(data);
                    if(ack) GLCD_Putchar('+');  // Ack
                    else    GLCD_Putchar('-');  // No Ack
                }
                GLCD_Putchar(' ');
            }
        }
        // Display UART or SPI data
        else {
            indrx=640-DMA.CH0.TRFCNT;
            if(testbit(MStatus, stop)) {
                lcd_line(64,8,64,30);
                lcd_goto(63,4); GLCD_Putchar('S');
                lcd_goto(63,5); GLCD_Putchar('T');
                lcd_goto(63,6); GLCD_Putchar('O');
                lcd_goto(63,7); GLCD_Putchar('P');
            }
            else lcd_line(64,8,64,64);
            lcd_goto(63,0); GLCD_Putchar(NibbleToChar(page));
            if(CHD.ctrl3.spi) {
                lcd_goto(60,1); GLCD_Putchar('I');
                lcd_goto(66,1); GLCD_Putchar('O');
            }
            else {
                lcd_goto(60,1); GLCD_Putchar('R');
                lcd_goto(66,1); GLCD_Putchar('T');
            }
            // Display RX or SDI data
            lcd_goto(1,0);
            for(i=(uint16_t)page*40; (i<((uint16_t)page+1)*40); i++) {
                if(i>=indrx) break;
                data=bigtemp[i];
                if(CHD.ctrl3.ascii) { GLCD_Putchar(data); GLCD_Putchar(' '); }
                else printhex(data);
                GLCD_Putchar(' ');
                if(u8CursorX>=60) {    // Next line
                    u8CursorX = 1; u8CursorY++;
                }
            }
            lcd_goto(72,0);
            // Display TX or SDO data
            for(i=(uint16_t)page*40; (i<((uint16_t)page+1)*40); i++) {
                if(i>=indtx) break;
                data=bigtemp[i+640];
                if(CHD.ctrl3.ascii) { GLCD_Putchar(data); GLCD_Putchar(' '); }
                else printhex(data);
                GLCD_Putchar(' ');
                if(u8CursorX<72) {    // Next line
                    u8CursorX = 72;
                }
            }
        }
        if(testbit(Signals, userinput)) {
            clrbit(Signals, userinput);
            if (key==KBC) CHD.ctrl3.ascii=!CHD.ctrl3.ascii;
            else if (key==KA) togglebit(MStatus, stop);
            else if (key==KB) {
                if(page) page--;
            }
            else if (key==KC) {
                if(page<15) page++;
            }
        }
    } while (key!=KD);
exit:
    USARTC0.CTRLA = 0x00;   // Disable receive interrupt
    USARTC0.CTRLB = 0x00;   // Disable UART RX
    TCC0.CTRLA = 0x00;      // Disable timer
    PORTC.INTCTRL = 0x00;   // Disable PORTC interrupts
    SPIC.CTRL = 0x00;       // Disable SPI
    PR.PRPC  = 0x7C;        // Stop: TWI, USART0, USART1, SPI, HIRES

    InitDMA();
}

#define SDA_PIN 0
#define SCL_PIN 1
#define SCL (unsigned char) (VPORT2.IN & _BV(SCL_PIN))
#define SDA (unsigned char) (VPORT2.IN & _BV(SDA_PIN))

// Decode I2C (TWI) protocol
// bigtemp: holds data
// CHD.data, first 128 hold if the byte is address or data
// CHD.data, last 128 hold if the byte had an ack
ISR(PORTC_INT0_vect) {
    uint8_t data;     // byte to accumulate data information
    uint8_t bits;     // current bit being processed
    uint8_t first=1;
    uint8_t *queue_max=bigtemp+1024;

    TCC1.CNT = 0;
    setbit(TCC1.INTFLAGS, TC1_OVFIF_bp);    // Clear timeout interrupt

    // wait for both high
    //while(SCL == 0 || SDA == 0);
    // wait for start condition
    while(SDA || SCL == 0) if(testbit(TCC1.INTFLAGS, TC1_OVFIF_bp)) return;  // SDA falls and SCL is still high
repeated_start:
    // wait for SCL to fall   
    while(SCL) if(testbit(TCC1.INTFLAGS, TC1_OVFIF_bp)) return;
    bits = 0;
sample_bit:
    // now ready for next bit of the byte
    while(SCL == 0) if(testbit(TCC1.INTFLAGS, TC1_OVFIF_bp)) return;  // look for rising  
    // sample data on SDA
    // we might not use this data in the cases of either
    // stop or repeated start conditions
    uint8_t dt = SDA;
    // SCL is now high
    // wait for either of two conditions: SCL falls or SDA changes
    // split the cases to reduce the sub-loop overheads
    if (dt == 0) {
        // special case where stop condition might exist
        // because data was zero
look_for_bit_or_stop:
        if (SCL == 0) goto process_bit;
        if (SDA == 0) goto look_for_bit_or_stop;
        goto stop_condition;
    }
    else {
        // special case where we might have a repeated start
        // where SDA fails while SCL high
look_for_bit_or_repeated_start:
        if(testbit(TCC1.INTFLAGS, TC1_OVFIF_bp)) return;    // timeout
        if (SCL == 0) goto process_bit;
        if (SDA) goto look_for_bit_or_repeated_start;
        first=1;
        goto repeated_start;
    }
process_bit:    
    // process another data bit
    dt = dt >> SDA_PIN;  // normalize the bit to the LSB position
    if (bits == 8) {
        // looking at the acknowledge bit
        if (queue_ptr == queue_max) goto queue_is_full;
        // writeQueue(data, dt);
        *queue_ptr++ = data;  // store the data
         indrx++;

        // acknowledge bits are accumulated into a byte to save memory
        // so, there are 8 data bytes before we have 1 acknowledge byte
        if(!dt) {   // Ack by pulling line low
            *ack_ptr |= ack_bit;
        }
        else {
            *ack_ptr &= ~ack_bit;
        }
        ack_bit = ack_bit>>1;
        if (ack_bit == 0) {  // 8 bits done
            ack_ptr++;
            ack_bit = 0x80;
        }
        bits = 0;  // next SCL rise starts a new byte of data

        // addr/data bits are accumulated into a byte to save memory
        // so, there are 8 data bytes before we have 1 byte  
        if(first) {
            *addr_ptr |= addr_bit;
        }
        else {
            *addr_ptr &= ~addr_bit;
        }
        addr_bit = addr_bit>>1;
        if (addr_bit == 0) {  // 8 bits done
            addr_ptr++;
            addr_bit = 0x80;
        }
        first=0;    // subsequent bytes are data bytes
    } 
    else {
        // accumulate bits of the byte's information (MSB to LSB)
        data = (data << 1) | dt;
        bits++;
    }
    goto sample_bit;

stop_condition:
    if (queue_ptr == queue_max) goto queue_is_full;
    PORTC.INTFLAGS = 0x01;  // Clear rising edge interrupt
    return;    //writeMessageSize();

    // the program comes here when the memory is full
queue_is_full:
    // If single trigger, now go to stop
    if(testbit(MStatus, single)) setbit(MStatus, stop);
    indrx=0;
    addr_ptr=CHD.data;
    ack_ptr=CHD.data+128;       // pointer to acknowledge bytes (packed bits)
    queue_ptr=bigtemp;          // pointer to data
    //setupReadAck();  // clean up the acknowledge byte
}

// Decode SPI protocol
// bigtemp: holds data
ISR(PORTC_INT1_vect) {
    uint8_t data, i=8;

    TCC1.CNT = 0;
    setbit(TCC1.INTFLAGS, TC1_OVFIF_bp);    // Clear timeout interrupt

    if(CHD.ctrl2.CPHA) {    // CPHA == 1
        do {
            // wait for clock rising
            while(!testbit(VPORT2.IN, 7)) if(testbit(TCC1.INTFLAGS, TC1_OVFIF_bp)) return;
            data=data<<1;
            // wait for clock falling
            while(testbit(VPORT2.IN, 7)) if(testbit(TCC1.INTFLAGS, TC1_OVFIF_bp)) return;
            if(testbit(VPORT2.IN, 6)) data|=0x01; // Read MISO
        } while(--i);

    }
    else {                  // CPHA == 0
        do {
            // wait for clock rising
            while(!testbit(VPORT2.IN, 7)) if(testbit(TCC1.INTFLAGS, TC1_OVFIF_bp)) return;
            data=data<<1;
            if(testbit(VPORT2.IN, 6)) data|=0x01; // Read MISO
            // wait for clock falling
            while(testbit(VPORT2.IN, 7)) if(testbit(TCC1.INTFLAGS, TC1_OVFIF_bp)) return;
        } while(--i);
    }
    PORTC.INTFLAGS = 0x02;  // Clear rising edge interrupt
    // Save data on buffer
    if(indtx>=640) indtx=0;
    bigtemp[640+indtx++]=data;
}

// Bit banging to decode the TX pin:
// The interrupt monitors the TX line at four times the baud rate,
// when the start bit is sensed, the timer period is set to the baud rate
// then all the incoming characters are processed inside the interrupt.
// Before exiting, the timer is set to 4 times the baud rate again.
ISR(TCC0_OVF_vect) {
    uint8_t i, data=0, paritybit=0;
    
    // Start Bit
    if(testbit(VPORT2.IN, 3)) return;
    TCC0.INTFLAGS = 0x01;   // Clear overflow flag
    while(!testbit(TCC0.INTFLAGS,0));   // wait for next time slice
    // Set actual baudrate
    TCC0.PER = baud;

    // Data bits
    for(i=0; i<8; i++) {
        TCC0.INTFLAGS = 0x01;   // Clear overflow flag
        while(!testbit(TCC0.INTFLAGS,0));   // wait for next time bit
        data=data>>1;
        if(testbit(VPORT2.IN, 3)) {
            data|=0x80;
            paritybit++;
        }
    }
    
    // Check Parity
    if(CHD.ctrl2.parmode) {
        TCC0.INTFLAGS = 0x01;   // Clear overflow flag
        while(!testbit(TCC0.INTFLAGS,0));   // wait for next time bit
        if(CHD.ctrl2.parity) {  // Odd Parity
            if( (testbit(paritybit,0) && testbit(VPORT2.IN, 3)) ||
                (!testbit(paritybit,0) && !testbit(VPORT2.IN, 3)))
                goto exituart;   // Parity error
        }
        else {                  // Even Parity
            if( (!testbit(paritybit,0) && testbit(VPORT2.IN, 3)) ||
                (testbit(paritybit,0) && !testbit(VPORT2.IN, 3)))
                goto exituart;   // Parity error
        }
    }

    // First Stop bit
    TCC0.INTFLAGS = 0x01;   // Clear overflow flag
    while(!testbit(TCC0.INTFLAGS,0));   // wait for next time bit
    if(!testbit(VPORT2.IN, 3))
        goto exituart;       // stop bit error

    // Second Stop bit
    if(CHD.ctrl2.stopbit) {
        TCC0.INTFLAGS = 0x01;   // Clear overflow flag
        while(!testbit(TCC0.INTFLAGS,0));   // wait for next time bit
        if(!testbit(VPORT2.IN, 3))
            goto exituart;  // stop bit error
    }

    if(indtx>=640) indtx=0;
    bigtemp[640+indtx++]=data;

exituart:
    // Idle bit
    TCC0.INTFLAGS = 0x01;   // Clear overflow flag
    while(!testbit(TCC0.INTFLAGS,0));   // wait for next time bit
    TCC0.PER = baud/4;
    TCC0.INTFLAGS = 0x01;   // Clear overflow flag
}
